#!/usr/bin/env python
#coding=utf8
# Copyright (C) 2015, Alibaba Cloud Computing

#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

#The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import os
import sys
import time
import logging
import logging.handlers
import hashlib
import base64
from optparse import OptionParser
from mns_http import MNSHttp,Request,Response
from common_tool import XMLTool

CMD_LIST = [["notifymsg_withauth", 
             "--host=http(s)://xxx",
             "模拟notify message，包含验权信息，除host不可指定任何参数，用于测试验权逻辑"],
             ["notifymsg_withoutauth", 
             "--host=http(s)://xxx [--uri=xxx] [--body=xxx] [--base64=true/false] [--format=xml/simplfied] [--topicowner=xxx] [--topicname=xxx] [--subscriber=xxx] [--subname=xxx] [--pubtime=xxx]", 
             "模拟notify message，不包含验权信息，支持指定body等多个参数，用于测试消息处理逻辑"]]
CMD_RULES = dict([[item[0], item[1]] for item in CMD_LIST])

#init logger
logger = logging.getLogger("mns_topic_tool")
file_handler = logging.handlers.RotatingFileHandler("mns_topic_tool.log", maxBytes=1*1024*1024, backupCount=3)
formatter = logging.Formatter('[%(asctime)s] [%(name)s] [%(levelname)s] [%(filename)s:%(lineno)d] [%(thread)d] %(message)s')
file_handler.setFormatter(formatter)
logger.addHandler(file_handler)
logger.setLevel(logging.INFO)

USER_AGENT = "Aliyun Notification Service Agent"
MNS_VERSION = "2015-06-06"

DEFAULT_REQID = "566FD963B2B71C44481F701A"
DEFAULT_MESSAGE = "TestMessage"
DEFAULT_MESSAGEID = "6CC4D900CA59A2CD-1-15180534A8F-200000001"
DEFAULT_TOPICOWNER = "TestTopicOwner"
DEFAULT_TOPICNAME = "TestTopicName"
DEFAULT_SUBSCRIBER = "TestSubscriber"
DEFAULT_SUBNAME = "TestSubscriptionName"

class MNSTopicTool:
    def notifymsg_withoutauth(self, args, options):
        MNSTopicTool.miss_parameters(options.host is None, args)
        MNSTopicTool.invalid_parameters(not options.host.startswith("http://") and not options.host.startswith("https://"), args, options.host)
        uri = "/notifications" if options.uri is None else options.uri
        encode_base64 = False if options.base64 is None or options.base64.lower() == "false" else True
        if options.host.startswith("http://"):
            host = options.host[7:]
            is_https = False
        elif options.host.startswith("https://"):
            host = options.host[8:]
            is_https = True

        #init request body
        data_dict = {"MessageId" : DEFAULT_MESSAGEID}
        data_dict["Message"] = DEFAULT_MESSAGE if options.body is None else options.body
        if encode_base64:
            data_dict["Message"] = base64.b64encode(data_dict["Message"])
        data_dict["TopicOwner"] = DEFAULT_TOPICOWNER if options.topicowner is None else options.topicowner
        data_dict["TopicName"] = DEFAULT_TOPICNAME if options.topicname is None else options.topicname
        data_dict["Subscriber"] = DEFAULT_SUBSCRIBER if options.subscriber is None else options.subscriber
        data_dict["SubscriptionName"] = DEFAULT_SUBNAME if options.subname is None else options.subname
        data_dict["PublishTime"] = str(int(time.time())) if options.pubtime is None else options.pubtime
        data_dict["MessageMD5"] = base64.b64encode(hashlib.md5(data_dict["Message"]).hexdigest())
        if options.format is None or options.format.lower() == "xml":
            data = XMLTool.dict_to_xml("Notification", data_dict, "http://mns.aliyuncs.com/doc/v1/")
        else:
            data = data_dict["Message"]

        #init request header
        header = {
            "Host" : host,
            "Date" : time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime()),
            "Content-Length" : len(data),
            "Content-Type" : "text/xml;charset=utf-8",
            "Content-MD5" : base64.b64encode(hashlib.md5(data).hexdigest()),
            "User-Agent" : USER_AGENT,
            "x-mns-version" : MNS_VERSION,
            "x-mns-request-id" : DEFAULT_REQID
        }

        #send request
        req = Request("POST", uri, header, data)
        resp = self.send_request(host, req, is_https, args[0])
    
    def notifymsg_withauth(self, args, options):
        MNSTopicTool.miss_parameters(options.host is None, args)
        MNSTopicTool.invalid_parameters(not options.host.startswith("http://") and not options.host.startswith("https://"), args, options.host)
        if options.host.startswith("http://"):
            host = options.host[7:]
            is_https = False
        elif options.host.startswith("https://"):
            host = options.host[8:]
            is_https = True

        #init request body
        data = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><Notification xmlns=\"http://ns.aliyuncs.com/doc/v1/\"><TopicOwner>1365541074971606</TopicOwner><TopicName>night</TopicName><Subscriber>1365541074971606</Subscriber><SubscriptionName>nightsub</SubscriptionName><MessageId>9CD0D085A77BDDF3-1-152A6161A8C-200000001</MessageId><MessageMD5>444D0565E9FA6CA49AE306E304DF5917</MessageMD5><Message>nightleslie</Message><PublishTime>1454485412492</PublishTime></Notification>"

        #init request header
        header = {
            "Host" : host,
            "Date" : "Wed, 03 Feb 2016 07:43:32 GMT",
            "Content-Length" : "451",
            "Content-Type" : "text/xml;charset=utf-8",
            "Content-MD5" : "YjY0NGZiNjE0YjFlNDQ0MWJmNjdlNWJhMzViMzViZDg=",
            "User-Agent" : "Aliyun Notification Service Agent",
            "x-mns-version" : "2015-06-06",
            "x-mns-request-id" : "56B1AFA4B3117F9E3F0464F3",
            "Authorization" : "cPpmTiRTv6GrBHKviLfBdq23A4hmIvK+h2665WgxPgu7mkrrzOuCnaDFUy1g2DKjQ/+jxbBY1otjbsJZomYyHg==",
            "x-mns-signing-cert-url" : "aHR0cDovL21uc3Rlc3Qub3NzLWNuLWhhbmd6aG91LmFsaXl1bmNzLmNvbS94NTA5X3B1YmxpY19jZXJ0aWZpY2F0ZS5wZW0="
        }

        #send request
        req = Request("POST", "/notifications", header, data)
        resp = self.send_request(host, req, is_https, args[0])

    def send_request(self, host, req, is_https, command):
        print "%sSend Request%s\n%s" % ("="*10, "="*10, req)
        logger.info("Request\n%s" % req)
        try:
            resp = MNSHttp.send_request(host, req, is_https)
        except Exception,e:
            print e
            MNSTopicTool.cmd_exception(command, e)
        print "%sGet Response%s\n%s" % ("="*10, "="*10, resp)
        logger.info("Response\n%s" % resp)
        return resp

    @staticmethod
    def miss_parameters(condition, args):
        if condition:
            print "%s miss parameters." % args[0]
            print_help(args[0])

    @staticmethod
    def invalid_parameters(condition, args, value):
        if condition:
            print "'%s' is invalid." % value
            print_help(args[0])

    @staticmethod
    def cmd_exception(action, e):
        print "%s fail!\nException:%s" % (action, e)
        sys.exit(-1)

    @staticmethod
    def cmd_fail(action, info):
        print "%s fail!\nInfo:%s" % (action, info)
        sys.exit(-1)

def print_help(key=None):
    print 80*"="
    if key is None:
        print "%sCommands%s\n" % (30*"-", 30*"-")
        print "\n".join(["%s  %s\n    %s\n" % (item[0].ljust(30),item[2],item[1]) for item in CMD_LIST])
    else:
        print "%s helpinfo:\n%s" % (key, CMD_RULES[key])
        print "\n%s" % (80*"=")
    sys.exit(1)

if __name__ == "__main__":
    parser = OptionParser('''%prog args [options]\n\nargs:\nnotifymsg_withoutauth''')
    parser.add_option("-i", "--info", action="store_true", dest="info", help="get more information of an action")
    parser.add_option("-H", "--host", dest="host", help="specify the host of subscription endpoint, such as: http://www.xxx-company.com")
    parser.add_option("-u", "--uri", dest="uri", help="specify the uri of subscription endpoint, default: /notifications")
    parser.add_option("-f", "--format", dest="format", help="specify the format of http request body, value: xml/simplified, default: xml")
    parser.add_option("-b", "--body", dest="body", help="specify the body of message, default: TestMessage")
    parser.add_option("-t", "--topicowner", dest="topicowner", help="specify topic owner, default: TestTopicOwner")
    parser.add_option("-T", "--topicname", dest="topicname", help="specify topic name, default: TestTopic")
    parser.add_option("-s", "--subscriber", dest="subscriber", help="specify subscriber name, default: TestSubscriber")
    parser.add_option("-S", "--subname", dest="subname", help="specify subscription name, default: TestSubscriptionName")
    parser.add_option("-p", "--pubtime", dest="pubtime", help="specify publish time, default get current time")
    parser.add_option("-B", "--base64", dest="base64", help="specify encode message body with base64 or not, default: false")

    (options, args) = parser.parse_args()
    if (len(args)) < 1:
        print_help()

    mns_topic_tool = MNSTopicTool()
    if args[0] not in CMD_RULES.keys():
        print "unsupported command : %s" % args[0]
        print "use --help for more information"
        sys.exit(1)

    if options.info:
        print_help(args[0])

    getattr(mns_topic_tool, args[0])(args, options)

